@@ -3,7 +3,7 @@
 ##
 ##    Functions for generating random point patterns
 ##
-##    $Revision: 4.121 $   $Date: 2024/08/11 02:36:43 $
+##    $Revision: 4.129 $   $Date: 2025/09/11 22:42:45 $
 ##
 ##    runifpoint()      n i.i.d. uniform random points ("binomial process")
 ##    runifdisc()       special case of disc (faster)
@@ -36,7 +36,8 @@
 ##          plot(rMaternII(100, 0.05))
 ##
 
-runifdisc <- function(n, radius=1, centre=c(0,0), ..., nsim=1, drop=TRUE)
+runifdisc <- function(n, radius=1, centre=c(0,0), ..., nsim=1, drop=TRUE,
+                      boxed=FALSE)
 {
   ## i.i.d. uniform points in the disc of radius r and centre (x,y)
   check.1.real(radius)
@@ -45,30 +46,50 @@ runifdisc <- function(n, radius=1, centre=c(0,0), ..., nsim=1, drop=TRUE)
     check.1.integer(nsim)
     stopifnot(nsim >= 0)
   }
-  disque <- disc(centre=centre, radius=radius, ...)
+  ensure2vector(centre)
+  if(boxed) {
+    ## replace disc by square
+    disque <- owin(centre[1] + radius * c(-1,1),
+                   centre[2] + radius * c(-1,1))
+  } else {
+    ## usual case: create disc window
+    disque <- do.call(disc,
+                      resolve.defaults(list(centre=centre,
+                                            radius=radius),
+                                       list(...),
+                                       list(npoly=32)))
+  }
+  ## constants
   twopi <- 2 * pi
   rad2 <- radius^2
+  nn <- nsim * n
+  ## generate all coordinates
+  theta <- matrix(runif(nn, min=0, max=twopi), n, nsim)
+  s     <- matrix(sqrt(runif(nn, min=0, max=rad2)), n, nsim)
+  xx <- centre[1] + s * cos(theta)
+  yy <- centre[2] + s * sin(theta)
+  ## pack up
   result <- vector(mode="list", length=nsim)
-  for(isim in seq_len(nsim)) {
-    theta <- runif(n, min=0, max=twopi)
-    s <- sqrt(runif(n, min=0, max=rad2))
-    result[[isim]] <- ppp(centre[1] + s * cos(theta),
-                          centre[2] + s * sin(theta),
+  for(jsim in seq_len(nsim)) {
+    result[[jsim]] <- ppp(xx[, jsim],
+                          yy[, jsim], 
                           window=disque, check=FALSE)
   }
   result <- simulationresult(result, nsim, drop)
   return(result)
 }
 
-
 runifpoint <- function(n, win=owin(c(0,1),c(0,1)),
                        giveup=1000, warn=TRUE, ...,
+                       fail.action=c("error", "pass", "missing"),
                        nsim=1, drop=TRUE, ex=NULL)
 {
   if(!missing(nsim)) {
     check.1.integer(nsim)
     stopifnot(nsim >= 0)
   }
+
+  fail.action <- match.arg(fail.action)
   
   if(missing(n) && missing(win) && !is.null(ex)) {
     stopifnot(is.ppp(ex))
@@ -165,8 +186,13 @@ runifpoint <- function(n, win=owin(c(0,1),c(0,1)),
                  break
                } else if(ntries >= giveup) {
                  ## otherwise get bored eventually
-                 stop(paste("Gave up after", giveup * n, "trials,",
-                            X$n, "points accepted"))
+                 switch(fail.action,
+                        pass    = { result[[isim]] <- X },
+                        missing = { result[[isim]] <- NAobject("ppp") },
+                        error   = {
+                           stop(paste("Gave up after", giveup * n, "trials,",
+                                     X$n, "points accepted"))
+                        })
                }
              }
            }
@@ -220,12 +246,17 @@ runifpoispp <- function(lambda, win = owin(c(0,1),c(0,1)), ...,
 }
 
 rpoint <- function(n, f, fmax=NULL,
-                   win=unit.square(), ..., giveup=1000, warn=TRUE, verbose=FALSE,
+                   win=unit.square(), ..., giveup=1000,
+                   fail.action=c("error", "pass", "missing"),
+                   warn=TRUE, verbose=FALSE,
                    nsim=1, drop=TRUE, forcewin=FALSE) {
+
+  fail.action <- match.arg(fail.action)
   
   if(missing(f) || (is.numeric(f) && length(f) == 1))
     ## uniform distribution
-    return(runifpoint(n, win, giveup=giveup, warn=warn, nsim=nsim, drop=drop))
+    return(runifpoint(n, win, giveup=giveup, fail.action=fail.action,
+                      warn=warn, nsim=nsim, drop=drop))
   
   ## non-uniform distribution....
   if(!is.function(f) && !is.im(f))
@@ -292,8 +323,20 @@ rpoint <- function(n, f, fmax=NULL,
           edgy <- edgy[!inside.owin(xe, ye, win.out)]
           if(ntries > giveup) break;
         }
+        accepted <- (length(edgy) == 0)
+      } else {
+        accepted <- TRUE
       }
-      result[[isim]] <- ppp(x, y, window=win.out, check=FALSE)
+      X <- ppp(x, y, window=win.out, check=FALSE)
+      if(!accepted)
+        switch(fail.action,
+               pass    = { },
+               missing = { X <- NAobject("ppp") },
+               error   = {
+                 stop(paste("Gave up after", giveup, "trials,",
+                            X$n, "points accepted"))
+               })
+      result[[isim]] <- X
     }
     result <- simulationresult(result, nsim, drop)
     return(result)
@@ -365,9 +408,15 @@ rpoint <- function(n, f, fmax=NULL,
           }
         }
       }
-      if(ntries > giveup)
-        stop(paste("Gave up after",giveup * n,"trials with",
-                   X$n, "points accepted"))
+      if(ntries > giveup) {
+        switch(fail.action,
+               pass    = { result[[isim]] <- X },
+               missing = { result[[isim]] <- NAobject("ppp") },
+               error   = {
+                 stop(paste("Gave up after",giveup * n,"trials with",
+                            X$n, "points accepted"))
+               })
+      }
     }
   }
   result <- simulationresult(result, nsim, drop)
@@ -388,8 +437,17 @@ rpoispp <- function(lambda, lmax=NULL, win = owin(), ...,
     check.1.integer(nsim)
     stopifnot(nsim >= 0)
   }
+
+  if(!missing(lambda) && inherits(lambda, "tessfun")) {
+    ## intensity is constant on each tile of a tessellation
+    ## use special algorithm
+    if(missing(win)) win <- NULL
+    result <- rpoisTessfun(lambda, win=win, nsim=nsim, drop=drop, ...)
+    return(result)
+  }
   
   if(missing(lambda) && is.null(lmax) && missing(win) && !is.null(ex)) {
+    ## use example pattern 'ex'
     lambda <- intensity(unmark(ex))
     win <- Window(ex)
   } else {
@@ -494,6 +552,58 @@ rpoispp <- function(lambda, lmax=NULL, win = owin(), ...,
   stop(paste(sQuote("lambda"), "must be a constant, a function or an image"))
 }
     
+rpoisTessfun <- function(lambda, win=NULL, ..., nsim=1, drop=TRUE,
+                         tilewise=NULL, verbose=TRUE) {
+  ## special case of rpoispp: constant intensity on each tile of tessellation
+  stopifnot(inherits(lambda, "tessfun"))
+  tes <- as.tess(lambda)
+  lam <- as.numeric(tessfunvalues(lambda))
+  lmax <- max(lam)
+  Wfull <- Window(as.tess(lambda))
+  if(is.null(win)) win <- Wfull else stopifnot(is.owin(win))
+  if(length(tilewise) == 0) {
+    ## decide which algorithm to use
+    switch(tes$type,
+           rect = ,
+           tiled = {
+             En <- integral(lambda)
+             Em <- lmax * area(Wfull)
+             tilewise <- (Em > .Machine$integer.max) || (En < Em/1e3)
+             if(verbose)
+               message("Using ", if(tilewise) "tilewise" else "rejection",
+                       " algorithm")
+           },
+           image = {
+             ## No benefit in splitting by tiles
+             tilewise <- FALSE
+           })
+  }
+  if(isFALSE(tilewise)) {
+    ## use rejection algorithm 
+    class(lambda) <- setdiff(class(lambda), "tessfun")
+    result <- rpoispp(lambda, lmax=lmax, win=win, ..., nsim=nsim, drop=drop)
+    return(result)
+  }
+  B <- Frame(win)
+  til <- tiles(tes)
+  nt <- length(til)
+  for(j in seq_len(nt)) {
+    ## generate 'nsim' patterns in tile 'j'
+    Uj <- runifpoispp(lam[j], til[[j]], nsim=nsim, drop=FALSE)
+    ## extract coordinates as list(list(x,y), list(x,y), ...)
+    if(j == 1) {
+      xylist <- lapply(Uj, concatxy)
+    } else {
+      xylist <- mapply(concatxy, xylist, Uj, SIMPLIFY=FALSE)
+    }
+  }
+  ## create point pattern objects
+  result <- lapply(xylist, as.ppp, W=B, check=FALSE)
+  ## clip to 'win'
+  result <- lapply(result, "[", i=win)
+  return(simulationresult(result, nsim, drop))
+}
+
 rMaternI <- function(kappa, r, win = owin(c(0,1),c(0,1)), stationary=TRUE,
                      ..., nsim=1, drop=TRUE)
 {
